// There are two model modes...
// 1. MODELMODE_LINK_SELECTED will always use the currently selected model.
const MODELMODE_LINK_SELECTED = 1
// 2. MODELMODE_FIXED has a fixed model, which it always uses.
const MODELMODE_FIXED         = 2

function GetGlyphGetter(obj)
{
  return function()
  {
    var idcr = Components.interfaces.IDataToCanvasRequest;
    if (obj.drawType == 'L')
      return idcr.GLYPH_LINES;
    else if (obj.drawType == 'SQ')
      return idcr.GLYPH_SQUARES;
    else if (obj.drawType == 'C')
      return idcr.GLYPH_CIRCLES;
    else if (obj.drawType == 'DI')
      return idcr.GLYPH_DIAMONDS;
    else if (obj.drawType == 'D')
      return idcr.GLYPH_DOTS;
    // Shouldn't be reached...
    alert("Unknown glyph string; this is a bug.");
    return -1;
  }
}

function GetDrawColourGetter(obj)
{
  return function()
  {
    return obj.getColourString();
  }
}

function GraphLineInformation(grapharea, hints)
{
  grapharea.changeBegin();

  if (window.unfreezeListeners == null)
    window.unfreezeListeners = [];
  window.unfreezeListeners.push(this);

  if (hints == null)
    hints = {};
  if (window.gliUniq == null)
    window.gliUniq = 1;
  this.gliUniq = window.gliUniq;
  window.gliUniq++;

  this.rdfresource = window.rdfservice.GetAnonymousResource();
  this.grapharea = grapharea;
  this.renderContext = grapharea.gcontext;
  if (hints.colour != null)
    this.setColour(hints.colour);
  else
    this.colour = parseInt(Math.random() * (1<<24));
  this.drawType = 'L';
  this.__defineGetter__("glyph", GetGlyphGetter(this));
  this.__defineGetter__("drawColour", GetDrawColourGetter(this));
  this.firstPoint = 0;

  this.x = this.y = null;
  this.modelMode = null;
  this.setLink();

  this.xVariableMin = -1.0;
  this.yVariableMin = -1.0;
  this.xVariableMax = 1.0;
  this.yVariableMax = 1.0;
  this.xMultiplier = 1.0;
  this.xOffset = 0.0;
  this.yMultiplier = 1.0;
  this.yOffset = 0.0;
  this.lastPoint = 0x3FFFFFFF;
  this.xVariable = null;
  this.yVariable = null;
  grapharea.changeEnd();
}

GraphLineInformation.prototype =
{
  QueryInterface: function(aIID)
  {
    if (aIID.equals(Components.interfaces.IDataToCanvasRequest) ||
        aIID.equals(Components.interfaces.nsISupports))
      return this;
    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  getColourString: function()
  {
    var c = this.colour;
    var cs = "";
    var i;
    for (i = 0; i < 6; i++)
    {
      cs = "0123456789abcdef"[c & 0xF] + cs;
      c >>= 4;
    }
    return "#" + cs;
  },

  setColour: function(c)
  {
    this.colour = parseInt(c.substr(1), 16);
  },

  unfrozen: function()
  {
    if (this.currentModel != null)
    {
      var dc = EnsureDataCollector(this.currentModel.model);
      dc.removeDataCollectorObserver(this.grapharea);
      dc.addDataCollectorObserver(this.grapharea);
    }
    this.switchModels(this.currentModel);
  },

  setLink: function()
  {
    if (this.modelMode == MODELMODE_LINK_SELECTED)
      return true;

    this.modelMode = MODELMODE_LINK_SELECTED;

    var cm = window.modelSelector.currentModel;
    if (cm == null)
      this.clearModel();
    else
      this.switchModels(cm);

    return true;
  },

  setDrawType: function(type)
  {
    this.drawType = type;
  },

  copySelectedModel: function()
  {
    this.grapharea.changeBegin();
    this.clearModel();
    this.modelMode = MODELMODE_FIXED;
    var cm = window.modelSelector.currentModel;
    if (cm == null)
    {
      alert("No model is currently selected.");
      this.grapharea.changeEnd();
      return false;
    }
    this.switchModels(cm);
    this.grapharea.changeEnd();

    return true;
  },

  // Internal API (externally use setLink and copySelectedModel).
  switchModels: function(model)
  {
    this.grapharea.changeBegin();
    this.clearModel();
    this.currentModel = model;

    this.datacollector = EnsureDataCollector(this.currentModel.model);

    // See if variable changes...
    if (this.xcomp != null)
    {
      var c = this.currentModel.model.allComponents.
        getComponent(this.xcomp);
      if (c == null)
        this.x = null;
      else
      {
        var v = c.variables.getVariable(this.xvar);
        if (v == null)
          this.x = null;
        else
          this.x = v;
      }
    }
    if (this.ycomp != null)
    {
      var c = this.currentModel.model.allComponents.
        getComponent(this.ycomp);
      if (c == null)
        this.y = null;
      else
      {
        var v = c.variables.getVariable(this.yvar);
        if (v == null)
          this.y = null;
        else
          this.y = v;
      }
    }

    // Now set the X and Y labels...
    if (this.xvar != null)
    {
      if (this.x == null)
        this.xlabel.setAttribute("label", this.xvar + " (not found)");
      else
        this.xlabel.setAttribute("label", this.xvar + " (" + this.x.unitsName +
                                 ")");
    }
    if (this.yvar != null)
    {
      if (this.y == null)
        this.ylabel.setAttribute("label", this.yvar + " (not found)");
      else
        this.ylabel.setAttribute("label", this.yvar + " (" + this.y.unitsName +
                                 ")");
    }

    this.recheckVariables();

    this.grapharea.addModelLine(this.currentModel.model, this);
    this.grapharea.changeEnd();
  },

  clearModel: function()
  {
    if (this.currentModel == null)
      return;

    this.grapharea.changeBegin();

    this.grapharea.removeModelLine(this.currentModel.model, this);

    EnsureDataCollector(this.currentModel.model).
      removeDataToCanvasRequest(this);

    // TODO: Deregister model handlers.
    this.currentModel = null;

    this.grapharea.changeEnd();
  },

  recheckVariables: function()
  {
    this.xVariable = this.yVariable = null;
    if (this.currentModel == null)
      return;
    var cm = GetTag(this.currentModel.model, "compiled_model");
    if (cm == null)
      return;
    cm = cm.compModel;
    this.integrationStarted(0, 0, 0, cm);
  },

  // External must call before abandaning this GraphLineInformation...
  deregister: function()
  {
    var i;
    for (i in window.unfreezeListeners)
      if (window.unfreezeListeners[i] == this)
        delete window.unfreezeListeners[i];
    this.clearModel();
  },

  // Callback hook, only to be called from the graph area.
  selected: function(newModel)
  {
    if (this.modelMode == MODELMODE_LINK_SELECTED)
      this.switchModels(newModel);
  },

  SetScale: function(xMin, xMax, yMin, yMax, mx, cx, my, cy)
  {
    this.xVariableMin = xMin;
    this.yVariableMin = yMin;
    this.xVariableMax = xMax;
    this.yVariableMax = yMax;
    this.xMultiplier = mx;
    this.xOffset = cx;
    this.yMultiplier = my;
    this.yOffset = cy;
  },

  integrationStarted: function(start, stop, increment, cmodel)
  {
    if (!CompareObjects(cmodel.model, this.currentModel.model))
      return;
    var ci = cmodel.codeInformation;
    var vi = ci.iterateVariables();
    var v;
    while ((v = vi.nextVariable()) != null)
    {
      if (CompareObjects(v.source, this.x))
      {
        if (v.type == 0/*BOUND*/)
          this.xVariable = ci.variableCount;
        else if (v.type == 1 || v.type == 2)
          return; // ignore the variable, it doesn't vary.
        else
          this.xVariable = v.variableIndex;
      }
      if (CompareObjects(v.source, this.y))
      {
        if (v.type == 0/*BOUND*/)
          this.yVariable = ci.variableCount;
        else if (v.type == 1 || v.type == 2)
          return; // ignore the variable, it doesn't vary.
        else
          this.yVariable = v.variableIndex;
      }
    }
  },

  enabled: function()
  {
    if (this.currentModel == null ||
        this.x == null || this.y == null ||
        this.drawType == 'I' || this.xVariable == null ||
        this.yVariable == null)
      return false;
    return true;
  }
};

function GraphData()
{
}

GraphData.prototype = {
  set: function() {
  }
};

function GraphSet()
{
}

GraphSet.prototype = {
  graphRenamed: function() {
  }
};


